package com.qmth.teachcloud.task.job;

import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.threadPool.MyThreadPool;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.task.job.service.JobService;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.StringJoiner;

/**
 * pdf自动生成任务
 */
public class CreatePdfTaskJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(CreatePdfTaskJob.class);

    @Resource
    JobService jobService;

    @Resource
    LockService lockService;

    @Resource
    MyThreadPool myThreadPool;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) {
        if (lockService.trylock(LockType.CREATE_PDF, LockType.CREATE_PDF.name())) {
            try {
                jobService.createPdfTask();
                int rate = myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getQueue().size() + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getActiveCount();
                BigDecimal percent = new BigDecimal(rate).divide(new BigDecimal(myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getMaximumPoolSize()), 2, BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.PERCENT).setScale(2, BigDecimal.ROUND_HALF_UP);
                StringJoiner stringJoinerThreadPool = new StringJoiner("，");
                stringJoinerThreadPool.add("当前线程使用率：" + percent + "%");
                stringJoinerThreadPool.add("核心线程数：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getCorePoolSize())
                        .add("最大线程数：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getMaximumPoolSize())
                        .add("线程数量：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getPoolSize())
                        .add("队列数量：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getQueue().size())
                        .add("活跃数量：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getActiveCount())
                        .add("已完成数量：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getCompletedTaskCount())
                        .add("总共数量：" + myThreadPool.getThreadPoolTaskExecutor().getThreadPoolExecutor().getTaskCount());
                jobExecutionContext.setResult(stringJoinerThreadPool.toString());
            } catch (Exception e) {
                log.error(SystemConstant.LOG_ERROR, e);
            } finally {
                lockService.unlock(LockType.CREATE_PDF, LockType.CREATE_PDF.name());
            }
        }
    }
}
